@@ -346,7 +346,7 @@ void Dynamic::map_message(pTHX_ const Descriptor *descriptor, const string &perl
     if (descriptor_map.find(descriptor->full_name()) != descriptor_map.end())
         croak("Message '%s' has already been mapped", descriptor->full_name().c_str());
     if (options.use_bigints)
-        eval_pv("require Math::BigInt", 1);
+        load_module(PERL_LOADMOD_NOIMPORT, newSVpvs("Math::BigInt"), NULL);
     HV *stash = gv_stashpvn(perl_package.data(), perl_package.size(), GV_ADD);
     const MessageDef *message_def = def_builder.GetMessageDef(descriptor);
     if (is_map_entry(message_def, options.implicit_maps))
@@ -504,7 +504,6 @@ void Dynamic::map_service(pTHX_ const ServiceDescriptor *descriptor, const strin
     mapped_services.insert(descriptor->full_name());
     used_packages.insert(perl_package);
 
-    HV *stash = gv_stashpvn(perl_package.data(), perl_package.size(), GV_ADD);
     ServiceDef *service_def = new ServiceDef(descriptor->full_name());
 
     switch (options.client_services) {
@@ -535,9 +534,13 @@ void Dynamic::map_service_noop(pTHX_ const ServiceDescriptor *descriptor, const
 }
 
 void Dynamic::map_service_grpc_xs(pTHX_ const ServiceDescriptor *descriptor, const string &perl_package, const MappingOptions &options, ServiceDef *service_def) {
-    eval_pv(("package " + perl_package + ";\n" +
-             "use Grpc::Client::BaseStub;\n" +
-             "@ISA = qw(Grpc::Client::BaseStub);").c_str(), 1);
+    string isa_name = perl_package + "::ISA";
+    AV *isa = get_av(isa_name.c_str(), 1);
+    SV *base_stub_package = newSVpvs("Grpc::Client::BaseStub");
+
+    SvREFCNT_inc(base_stub_package);
+    av_push(isa, base_stub_package);
+    load_module(PERL_LOADMOD_NOIMPORT, base_stub_package, NULL);
 
     for (int i = 0, max = descriptor->method_count(); i < max; ++i) {
         const MethodDescriptor *method = descriptor->method(i);
