@@ -18,6 +18,8 @@
 namespace gpd {
 
 class Mapper;
+class MethodMapper;
+class ServiceDef;
 
 struct MappingOptions {
     enum AccessorStyle {
@@ -27,6 +29,12 @@ struct MappingOptions {
         Plain = 4,
     };
 
+    enum ClientService {
+        Disable = 0,
+        Noop = 1,
+        GrpcXS = 2,
+    };
+
     bool use_bigints;
     bool check_required_fields;
     bool explicit_defaults;
@@ -35,6 +43,7 @@ struct MappingOptions {
     bool generic_extension_methods;
     bool implicit_maps;
     AccessorStyle accessor_style;
+    ClientService client_services;
 
     MappingOptions(pTHX_ SV *options_ref);
 };
@@ -56,6 +65,7 @@ public:
     void map_package(pTHX_ const std::string &pb_package, const std::string &perl_package_prefix, const MappingOptions &options);
     void map_package_prefix(pTHX_ const std::string &pb_prefix, const std::string &perl_package_prefix, const MappingOptions &options);
     void map_enum(pTHX_ const std::string &enum_name, const std::string &perl_package, const MappingOptions &options);
+    void map_service(pTHX_ const std::string &service_name, const std::string &perl_package, const MappingOptions &options);
     void resolve_references();
 
     const Mapper *find_mapper(const upb::MessageDef *message_def) const;
@@ -69,6 +79,9 @@ private:
     void map_message_recursive(pTHX_ const google::protobuf::Descriptor *descriptor, const std::string &perl_package, const MappingOptions &options);
     void map_message(pTHX_ const google::protobuf::Descriptor *descriptor, const std::string &perl_package, const MappingOptions &options);
     void map_enum(pTHX_ const google::protobuf::EnumDescriptor *descriptor, const std::string &perl_package, const MappingOptions &options);
+    void map_service(pTHX_ const google::protobuf::ServiceDescriptor *descriptor, const std::string &perl_package, const MappingOptions &options);
+    void map_service_noop(pTHX_ const google::protobuf::ServiceDescriptor *descriptor, const std::string &perl_package, const MappingOptions &options, ServiceDef *service_def);
+    void map_service_grpc_xs(pTHX_ const google::protobuf::ServiceDescriptor *descriptor, const std::string &perl_package, const MappingOptions &options, ServiceDef *service_def);
     void check_package(pTHX_ const std::string &perl_package, const std::string &pb_name);
 
     OverlaySourceTree overlay_source_tree;
@@ -80,8 +93,10 @@ private:
     STD_TR1::unordered_map<std::string, const Mapper *> descriptor_map;
     STD_TR1::unordered_set<std::string> used_packages;
     STD_TR1::unordered_set<std::string> mapped_enums;
+    STD_TR1::unordered_set<std::string> mapped_services;
     STD_TR1::unordered_set<const google::protobuf::FileDescriptor *> files;
     std::vector<Mapper *> pending;
+    std::vector<MethodMapper *> pending_methods;
 };
 
 };
