@@ -7,7 +7,9 @@ use parent 'Test::Builder::Module';
 use Test::More;
 use Test::Differences;
 use Test::Exception;
+use Test::Warn;
 
+use t::lib::DummyTiedScalar;
 use t::lib::DummyTiedArray;
 use t::lib::DummyTiedHash;
 
@@ -18,8 +20,10 @@ our @EXPORT = (
     @Test::More::EXPORT,
     @Test::Differences::EXPORT,
     @Test::Exception::EXPORT,
+    @Test::Warn::EXPORT,
     qw(
           maybe_bigint
+          tie_scalar
           tied_array
           tied_hash
     )
@@ -50,6 +54,10 @@ sub maybe_bigint {
     return $bi > -2147483648 && $bi < 2147483647 ? 0 + $_[0] : $bi;
 }
 
+sub tie_scalar {
+    tie $_[0], 't::lib::DummyTiedScalar', \$_[1];
+}
+
 sub tied_array {
     my @result;
 
